/* ideal1.c -- Simplest linear version of Zeljka's model
 */

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "ideal1.h"

void ideal1(TLONG nth, TFLOAT dt, TFLOAT trop, TFLOAT skew, TFLOAT qscale,
	    TFLOAT d, TFLOAT pwatmean, TFLOAT mrelax, TFLOAT mucin,
	    TFLOAT epsilon, TFLOAT lambdas, TFLOAT delq, TFLOAT wscale,
	    TFLOAT ctherm, TFLOAT mu, TFLOAT lam,
	    TFLOAT vx0, TFLOAT *z, TFLOAT *th, TFLOAT *rt,
	    TFLOAT *pi, TFLOAT *vx, TFLOAT *vy, TFLOAT *eta, TFLOAT *sth,
	    TFLOAT *sthe, TFLOAT *srt, TFLOAT *fx, TFLOAT *fy,
	    TFLOAT *rain1, TFLOAT *rain2s, TFLOAT *rain2t, TFLOAT *rain,
	    TFLOAT *eflux, TFLOAT *rflux, TFLOAT *vxflux,
	    TFLOAT *vyflux, TFLOAT *thesmean)
{
  TLONG ith;
  TFLOAT m0, ltot, pwat, frac, thes2, thes1, thesd,
    thebl, ueff, bb, thsum, rtsum, sthform, dsig;

  /* compute constants */
  ltot = LVAPOR + LFREEZ;
  m0 = 3.14159/trop;
  dsig = 1./nth;

  /* compute precipitable water */
  pwat = 0.;
  for (ith = 0; ith < nth; ith++) {
    pwat += rt[ith]*eta[ith]*dsig;
  }

  /* compute the perturbation evaporation rate and other fluxes */
  ueff = vx0*(vx[0] - vx0)/pow(vx0*vx0 + wscale*wscale,0.5);
  rflux[0] = RHOREF*ctherm*delq*ueff;
  eflux[0] = (ltot/CP)*rflux[0];
  vxflux[0] = -lam*vx[0]/mu;
  vyflux[0] = -lam*vy[0]/mu;

  /* find the first z level above d and the fractional position */
  ith = 1;
  while (d > z[ith] && ith < nth - 1) {
    ith++;
  }
  frac = (d - z[ith - 1])/(z[ith] - z[ith - 1]);

  /* interpolate theta_es at height d and find theta_e at surface */
  thes2 = thetaescomp(th[ith],pi[ith]) - thesmean[ith];
  thes1 = thetaescomp(th[ith - 1],pi[ith - 1]) - thesmean[ith - 1];
  thesd = (1. - frac)*thes1 + frac*thes2;
  thebl = (lambdas/RHOREF)*eflux[0];

  /* compute the perturbation precipitation rate */
  /* rain and rflux have units of kg/m^2/s = mm/s */
  rain1[0] = mrelax*(pwat - pwatmean);
  rain2s[0] = mucin*(RHOREF*CP/ltot)*thebl;
  rain2t[0] = -mucin*(RHOREF*CP/ltot)*thesd;
  rain[0] = rain1[0] + rain2s[0] + rain2t[0];

  /* compute source term profiles */
  /* the units of sth are K/ks because eta is a factor of 1000 smaller */
  /* than the mks value due to g having the units of km/ks^2 */
  bb = ltot*(rain[0] + epsilon*rain1[0])/CP;
  thsum = rtsum = 0.;
  for (ith = 0; ith < nth; ith++) {
    sthform = sin(m0*z[ith])*exp(skew*(z[ith] - trop/2.));
    sth[ith] = z[ith] < trop ? sthform : 0.;
    srt[ith] = exp(-z[ith]/qscale);
    thsum += eta[ith]*sth[ith]*dsig;
    rtsum += eta[ith]*srt[ith]*dsig;
  }
  for (ith = 0; ith < nth; ith++) {
    sth[ith] = bb*sth[ith]/thsum;
    srt[ith] = (rflux[0] - rain[0])*srt[ith]/rtsum;
    sthe[ith] = sth[ith] + ltot*srt[ith]/CP;
    fx[ith] = mu*vxflux[0]*exp(-mu*z[ith]);
    fy[ith] = mu*vyflux[0]*exp(-mu*z[ith]);
  }
}
