/* index.h -- Indexing tools for step.c.
 */

#define BOUNDS 0

/* index checkers */
#if BOUNDS
#define I(ix,iy,ith) iii(nx,ny,nth,ix,iy,ith)
#define J(ix,iy,ith) jjj(nx,ny,nth,ix,iy,ith)
#define I2(ix,iy) ii(nx,ny,ix,iy)
#define J2(ix,iy) jj(nx,ny,ix,iy)
#define IY(iy) fiy(ny,iy)
#define JY(iy) fjy(ny,iy)
#define K(iy,ith) fk(ny,nth,iy,ith)
#define K2(ix,iy) fk2(nx,ny,ix,iy)
#else
#define I(ix,iy,ith) ((ith) + 1 + (nth + 2)*((iy) + 2 + (ny + 4)*((ix) + 2)))
#define J(ix,iy,ith) ((ith) + (nth + 1)*((iy) + (ny + 1)*(ix)))
#define I2(ix,iy) ((iy) + 2 + (ny + 4)*((ix) + 2))
#define J2(ix,iy) ((iy) + (ny + 1)*(ix))
#define IY(iy) ((iy) + 2)
#define JY(iy) (iy)
#define K(iy,ith) ((ith) + nth*(iy))
#define K2(ix,iy) ((iy) + ny*(ix))
#endif

/* prototypes of bounds checking functions */
int iii(int nx, int ny, int nth, int ix, int iy, int ith);
int jjj(int nx, int ny, int nth, int ix, int iy, int ith);
int ii(int nx, int ny, int ix, int iy);
int jj(int nx, int ny, int ix, int iy);
int fiy(int ny, int iy);
int fjy(int ny, int iy);
int fk(int ny, int nth, int iy, int ith);
int fk2(int nx, int ny, int ix, int iy);

/* prototype for vertical hypersmoother */
void vsmooth(int nth, double *fld, double lambda);

/* prototype for horizontal hypersmoother */
void hsmooth(int nx, int ny, int nth, int channel, double *fld, double kappa);

/* macros for averaging, differencing, etc. */
#define AVXY(ix,iy,ith,fld) (0.25*(fld[I(ix,iy,ith)] + fld[I(ix - 1,iy,ith)] + fld[I(ix,iy - 1,ith)] + fld[I(ix - 1,iy - 1,ith)]))

#define AVXTH(ix,iy,ith,fld) (0.25*(fld[I(ix,iy,ith)] + fld[I(ix - 1,iy,ith)] + fld[I(ix,iy,ith - 1)] + fld[I(ix - 1,iy,ith - 1)]))

#define AVYTH(ix,iy,ith,fld) (0.25*(fld[I(ix,iy,ith)] + fld[I(ix,iy - 1,ith)] + fld[I(ix,iy,ith - 1)] + fld[I(ix,iy - 1,ith - 1)]))

#define AV(ix,iy,ith,fld) (0.5*(AVXY(ix,iy,ith,fld) + AVXY(ix,iy,ith - 1,fld)))

#define DIVX(ix,iy,ith,fld) ((AVYTH(ix,iy,ith,fld) - AVYTH(ix - 1,iy,ith,fld))/dx)

#define DIVY(ix,iy,ith,fld) ((AVXTH(ix,iy,ith,fld) - AVXTH(ix,iy - 1,ith,fld))/dy)

#define DIVTH(ix,iy,ith,fld) ((AVXY(ix,iy,ith,fld) - AVXY(ix,iy,ith - 1,fld))/dsig)

#define AV2(ix,iy,fld) (0.25*(fld[I2(ix,iy)] + fld[I2(ix - 1,iy)] + fld[I2(ix,iy - 1)] + fld[I2(ix - 1,iy - 1)]))

#define XAVXY(ix,iy,ith,fld) (0.25*(fld[J(ix + 1,iy + 1,ith)] + fld[J(ix,iy + 1,ith)] + fld[J(ix + 1,iy,ith)] + fld[J(ix,iy,ith)]))

#define XAVXTH(ix,iy,ith,fld) (0.25*(fld[J(ix + 1,iy,ith + 1)] + fld[J(ix,iy,ith + 1)] + fld[J(ix + 1,iy,ith)] + fld[J(ix,iy,ith)]))

#define XAVYTH(ix,iy,ith,fld) (0.25*(fld[J(ix,iy + 1,ith + 1)] + fld[J(ix,iy + 1,ith)] + fld[J(ix,iy,ith + 1)] + fld[J(ix,iy,ith)]))

#define XAV(ix,iy,ith,fld) (0.5*(XAVXY(ix,iy,ith + 1,fld) + XAVXY(ix,iy,ith,fld)))

#define XDIVX(ix,iy,ith,fld) ((XAVYTH(ix + 1,iy,ith,fld) - XAVYTH(ix,iy,ith,fld))/dx)

#define XDIVY(ix,iy,ith,fld) ((XAVXTH(ix,iy + 1,ith,fld) - XAVXTH(ix,iy,ith,fld))/dy)

#define XDIVTH(ix,iy,ith,fld) ((XAVXY(ix,iy,ith + 1,fld) - XAVXY(ix,iy,ith,fld))/dsig)

#define ADVX(ix,iy,vx,th) (vx[I2(ix,iy)] > 0. ? vx[I2(ix,iy)]*(th[I2(ix,iy)] - th[I2(ix - 1,iy)])/dx : vx[I2(ix,iy)]*(th[I2(ix + 1,iy)] - th[I2(ix,iy)])/dx)

#define ADVY(ix,iy,vy,th) (vy[I2(ix,iy)] > 0. ? vy[I2(ix,iy)]*(th[I2(ix,iy)] - th[I2(ix,iy - 1)])/dy : vy[I2(ix,iy)]*(th[I2(ix,iy + 1)] - th[I2(ix,iy)])/dy)

#define AVY(iy,f) (0.5*(f[iy] + f[iy + 1]))

/* macros for computing grid point centers */
#define XC(ix) (dx*(((double)ix) + 0.5) + xbase)

#define YC(iy) (dy*(((double)iy) + 0.5) + ybase)

#define SIGC(ith) (dsig*((ith) + 0.5))

#define SURFEXTRAP(fld,ix,iy) (1.5*fld[I(ix,iy,0)] - 0.5*fld[I(ix,iy,1)])
