/* plist.c -- Read and manage a list of parameter-value pairs.
 */
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <quickcandis.h>
#include "params.h"

#define MAXPARAMS 200
#define MAXSTRING 80

static char names[MAXPARAMS][MAXSTRING];
static char values[MAXPARAMS][MAXSTRING];
static int nparams = 0;
static int find_name(char *pname);
static int isnumber(char *pval);

/* Get the parameters from a specified file. */
int params_read(char *file_name)
{
  FILE *infile;
  char line[MAXSTRING];

  /* open the file containing the parameters */
  if ((infile = fopen(file_name,"r")) == (FILE*)NULL) {
    fprintf(stderr,"Params: Cannot open file %s\n",file_name);
    exit(1);
  }

  /* read in the parameters and put thim in the string table */
  nparams = 0;
  while (fgets(line,MAXSTRING - 1,infile) != (char*)NULL) {

    /* dispense with comment */
    if (line[0] == '#') continue;

    /* brute force string hygiene */
    line[MAXSTRING - 1] = line[MAXSTRING - 2] = '\0';
    sscanf(line,"%s %s",names[nparams],values[nparams]);
    nparams++;

    /* don't overflow table */
    if (nparams >= MAXPARAMS) {
      fprintf(stderr,"Params: too many parameters\n");
      exit(1);
    }
  }

  /* close the file and return */
  fclose(infile);
  return(0);
}

/* Print out the parameters. */
void params_print()
{
  int i;
  for (i = 0; i < nparams; i++) {
    printf("%s = %s\n",names[i],values[i]);
  }
}

/* Return a specified parameter value as a string. */
char *params_string(char *pname)
{
  int pos;
  pos = find_name(pname);
  return(values[pos]);
}

/* Return a specified parameter value as an int. */
int params_int(char *pname)
{
  int pos;
  pos = find_name(pname);
  return(atoi(values[pos]));
}

/* Return a specified parameter value as a float. */
float params_float(char *pname)
{
  int pos;
  pos = find_name(pname);
  return(atof(values[pos]));
}

/* Find the location of a parameter specified by name. */
static int find_name(char *pname)
{
  int i;
  for (i = 0; i < nparams; i++) {
    if (strcmp(pname,names[i]) == 0) {
      return(i);
    }
  }
  fprintf(stderr,"Params: cannot find parameter name %s\n",pname);
  exit(1);
}

/* Output parameters to Candis file. */
void params_candis_out(int chandle)
{
  int i;
  char comment[MAXSTRING];

  /* loop through parameters, writing them to a Candis file as comments */
  for (i = 0; i < nparams; i++) {
    if (isnumber(values[i])) {
      candisparam(chandle,names[i],atof(values[i]));
    }
    else {
      sprintf(comment,"  %s = %s",names[i],values[i]);
      candiscomment(chandle,comment);
    }
  }
}

/* Checks to see if string is a number */
static int isnumber(char *pval)
{
  int i;
  char c;

  i = 0;
  while ((c = pval[i++]) != '\0') {
    if ((c == '+' || c == '-' || c == '.' || c == 'e' || 
	  (c >= '0' && c <= '9')) != 1) return(0);
  }
  return(1);
}
