/* sig2z.c -- Transform sigma coordinate files to z coordinate files */
#include <stdio.h>
#include <math.h>
#include "cdfhdr.h"

#define SIG "sigma"

static char hin[HBMAX][LINE],hout[HBMAX][LINE];
static char cline[LINE],pline[LINE];
static long nsbin,nsbout,nvbin,nvbout;
static float *sbin,*sbout,*vbin,*vbout;
static struct field *fp,*fpo,*fpz,sflds[HBMAX],vflds[HBMAX],oflds[HBMAX];
static long nsflds,nvflds,gotsig[HBMAX];

main(argc,argv)
int argc;
char *argv[];
{
  float z0,dz;
  float *fptr,*zptr,*inptr,*outptr;
  long loop,poop;
  long nsigma,nz,nreps;

/* check command line arguments */
  if (argc != 4) {
    fprintf(stderr,"Usage: sig2z z0 dz nz\n");
    exit(1);
  }
  z0 = atof(argv[1]);
  dz = atof(argv[2]);
  nz = atoi(argv[3]);

/* get the input header */
  gethdr(stdin,hin,HBMAX);
  if (getfmt(hin,HBMAX) != 'f') {
    fprintf(stderr,"sig2z: float format expected\n");
    exit(1);
  }

/* create input buffers */
  nsbin = elemcnt(hin,HBMAX,'s');
  sbin = getbuff(nsbin);
  nvbin = elemcnt(hin,HBMAX,'v');
  vbin = getbuff(nvbin);

/* read the static slice */
  getslice(stdin,nsbin,sbin);

/* create the output header */
  nullhdr(hout,HBMAX,"float");

/* copy the comments */
  copycmt(hout,HBMAX,hin,HBMAX);

/* add one of our own */
  sprintf(cline,"sig2z: %s %s %s\n",argv[1],argv[2],argv[3]);
  addcline(hout,HBMAX,cline);

/* copy parameters except for sigma index parameters -- replace those */
  loop = 0;
  while (getpar(hin,HBMAX,loop++,cline,pline) == OK) {
    if (strcmp(cline,"sigma0") == 0) {
      addpar(hout,HBMAX,"z0",argv[1]);
    }
    else if (strcmp(cline,"dsigma") == 0) {
      addpar(hout,HBMAX,"dz",argv[2]);
    }
    else {
      addpar(hout,HBMAX,cline,pline);
    }
  }

/* copy static field entries except for sigma index field -- replace this */
  nsflds = 0;
  while ((fp = getfld(hin,HBMAX,'s',nsflds)) != NULL) {
    if (strcmp(fp->fname,SIG) == 0) {
      addfld(hout,HBMAX,'s',"z",1000.,0.,'s',1,"z",nz);
    }
    else {
      addfld(hout,HBMAX,'s',fp->fname,1000.,0.,'s',fp->dim,fp->dname1,
	     fp->dsize1);
    }
    sflds[nsflds++] = *fp;
  }

/* copy variable field entries */
  nvflds = 0;
  while ((fp = getfld(hin,HBMAX,'v',nvflds)) != NULL) {
    gotsig[nvflds] = 1;
    if (fp->dim > 0 && strcmp(fp->dname1,SIG) == 0) {
      addfld(hout,HBMAX,'v',fp->fname,1000.,0.,'s',fp->dim,
	     "z",nz,
	     fp->dname2,fp->dsize2,
	     fp->dname3,fp->dsize3,
	     fp->dname4,fp->dsize4);
    }
    else if (fp->dim > 1 && strcmp(fp->dname2,SIG) == 0) {
      addfld(hout,HBMAX,'v',fp->fname,1000.,0.,'s',fp->dim,
	     fp->dname1,fp->dsize1,
	     "z",nz,
	     fp->dname3,fp->dsize3,
	     fp->dname4,fp->dsize4);
    }
    else if (fp->dim > 2 && strcmp(fp->dname3,SIG) == 0) {
      addfld(hout,HBMAX,'v',fp->fname,1000.,0.,'s',fp->dim,
	     fp->dname1,fp->dsize1,
	     fp->dname2,fp->dsize2,
	     "z",nz,
	     fp->dname4,fp->dsize4);
    }
    else if (fp->dim > 3 && strcmp(fp->dname4,SIG) == 0) {
      addfld(hout,HBMAX,'v',fp->fname,1000.,0.,'s',fp->dim,
	     fp->dname1,fp->dsize1,
	     fp->dname2,fp->dsize2,
	     fp->dname3,fp->dsize3,
	     "z",nz);
    }
    else {
      addfld(hout,HBMAX,'v',fp->fname,1000.,0.,'s',fp->dim,
	     fp->dname1,fp->dsize1,
	     fp->dname2,fp->dsize2,
	     fp->dname3,fp->dsize3,
	     fp->dname4,fp->dsize4);
      gotsig[nvflds] = 0;
    }
    vflds[nvflds] = *fp;
    if ((fpo = seekfld(hout,HBMAX,'v',fp->fname)) == NULL) {
      fprintf(stderr,"sig2z: oops on %s\n",fp->fname);
      exit(1);
    }
    oflds[nvflds++] = *fpo;
  }

/* create output buffers */
  nsbout = elemcnt(hout,HBMAX,'s');
  sbout = getbuff(nsbout);
  nvbout = elemcnt(hout,HBMAX,'v');
  vbout = getbuff(nvbout);

/* write output header */
  puthdr(stdout,hout,HBMAX);

/* transfer static fields and create a z index field */
  loop = 0;
  while (loop < nsflds) {
    if (strcmp(sflds[loop].fname,SIG) == 0) {
      fptr = getptr(hout,HBMAX,sbout,'s','d',"z");
      for (poop = 0; poop < nz; poop++) {
	fptr[poop] = poop*dz + z0;
      }
      nsigma = sflds[loop].tsize;
    }
    else {
      fptr = getptr(hout,HBMAX,sbout,'s','d',sflds[loop].fname);
      for (poop = 0; poop < sflds[loop].tsize; poop++) {
	fptr[poop] = sbin[sflds[loop].ptr + poop];
      }
    }
    loop++;
  }

/* write the static slice */
  putslice(stdout,nsbout,sbout);

/* get the z field pointer */
  if ((fpz = seekfld(hin,HBMAX,'v',"z")) == NULL) {
    fprintf(stderr,"sig2z: can't find z field in input\n");
    exit(1);
  }
  nreps = fpz->tsize/nsigma;

/* loop on variable slices */
  while (getslice(stdin,nvbin,vbin) != EOF) {

/* loop on fields, transferring data for each field */
    for (loop = 0; loop < nvflds; loop++) {
      if (gotsig[loop] && vflds[loop].dim == fpz->dim) {

/* this assumes that sigma is the last index */
	for (poop = 0; poop < nreps; poop++) {
	  zptr = &vbin[fpz->ptr + poop*nsigma];
	  inptr = &vbin[vflds[loop].ptr + poop*nsigma];
	  outptr = &vbout[oflds[loop].ptr + poop*nz];

/* do the interpolation */
	  dointerp(nsigma,nz,z0,dz,zptr,inptr,outptr);
	}
      }
      else {
	for (poop = 0; poop < vflds[loop].tsize; poop++) {
	  vbout[oflds[loop].ptr + poop] = vbin[vflds[loop].ptr + poop];
	}
      }
    }
    putslice(stdout,nvbout,vbout);
  }

/* normal exit */
  exit(0);
}

/* do the bloody interpolation */
dointerp(nsigma,nz,z0,dz,z,inptr,outptr)
long nsigma,nz;
float z0,dz;
float *z,*inptr,*outptr;
{
  long iz,isig;
  float zval,zfrac;
  for (iz = 0; iz < nz; iz++) {
    zval = z0 + iz*dz;
    isig = 1;
    while (isig < nsigma - 1 && z[isig] < zval) {
      isig++;
    }
    zfrac = (zval - z[isig - 1])/(z[isig] - z[isig - 1]);
    if (zfrac < 0.) zfrac = 0.;
    if (zfrac > 1.) zfrac = 1.;
    outptr[iz] = zfrac*inptr[isig] + (1. - zfrac)*inptr[isig - 1];
  }
}
