/* sigma.c -- Integrate sigma isentropic equations in a beta plane
 * channel model.  Now under version control by subversion.
 */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <fenv.h>
#include "params.h"
#include "grid.h"
#include "step.h"
#include "wallclock.h"


#define VERSION "sigma-112"

main(int argv, char *argc[])
{
  int it, jt, mt, nt, counter, ocounter, microscope, ranseed;
  double dt, time, begmic, endmic, begout;
  char outfilename[81];

  /* enable core dumps on floating point exceptions */
  feenableexcept(FE_DIVBYZERO | FE_INVALID | FE_OVERFLOW); 

  /* check arguments */
  if (argv != 2) {
    fprintf(stderr,"Usage: sigma parameter_file\n");
    exit(1);
  }

  /* read parameters */
  params_read(argc[1]);
  nt = params_int("nt");
  mt = params_int("mt");
  dt = params_float("dt");
  ranseed = params_int("ranseed");
  begmic = params_float("begmic");
  endmic = params_float("endmic");
  begout = params_float("begout");
  strncpy(outfilename,params_string("outfile"),80);
  outfilename[80] = '\0';
  time = 0.;

  /* seed the random number generator -- default value = 1 */
  srandom(ranseed);

  /* initialize fields */
  step_init(time);
  step_output(outfilename,0,time,VERSION);

  /* loop on output */
  counter = 0;
  ocounter = 0;
  for (it = 1; it <= nt; it++) {

    /* subloop on time stepping */
    for (jt = 0; jt < mt; jt++) {
      counter++;
      time = counter*dt;
      microscope = (time >= begmic) && (time < endmic);
      step_do(time);
      if (microscope) step_output(outfilename,++ocounter,time,VERSION);
    }
    if (!microscope && time >= begout) {
      step_output(outfilename,++ocounter,time,VERSION);
    }
  }
  exit(0);
}
