/* thermo.h -- Definition file for thermo.c and thermodynamic constants.
 */

/* set the types of variables -- this is here so that, for instance,
   all floats can be set to doubles, etc. */
#define TFLOAT double
#define TDOUBLE double
#define TINT int
#define TLONG int

/* thermodynamic constants */
#define GEE 9800.0
#define FREEZING 273.15
#define PREF 100000.0
#define TREF 300.0
#define KAPPA 0.286
#define CP 1005.0
#define RHOREF 1.2
#define LVAPOR 2.50e6
#define LFREEZ 3.33e5
#define ALFA ((LVAPOR + LFREEZ)/(CP*TREF))
#define EPSILON 0.622

TFLOAT rs(TFLOAT th, TFLOAT pi);
TFLOAT escalc(TFLOAT td);
TFLOAT thetacomp2(TFLOAT the, TFLOAT r, TFLOAT pi);
TFLOAT thetacomp(TFLOAT the, TFLOAT r, TFLOAT pi);
TFLOAT thetaecomp(TFLOAT th, TFLOAT r, TFLOAT pi);
TFLOAT thetaecomp2(TFLOAT th, TFLOAT r, TFLOAT rsat);
TFLOAT thetaescomp(TFLOAT th, TFLOAT pi);
TFLOAT gmoist(TFLOAT th, TFLOAT pi);
TFLOAT pifromp(TFLOAT p);
TFLOAT pfrompi(TFLOAT pi);
TFLOAT rhocomp(TFLOAT th, TFLOAT pi);
